==> mob_variants_api_zenxarch-0.0.1-sources.jar: org/codeberg/zenxarch/mob_variants_api/variant/MobVariant.java <==
package org.codeberg.zenxarch.mob_variants_api.variant;

import com.mojang.serialization.Codec;
import com.mojang.serialization.DataResult;
import com.mojang.serialization.codecs.RecordCodecBuilder;
import java.util.List;
import java.util.Map;
import net.fabricmc.fabric.api.attachment.v1.AttachmentType;
import net.fabricmc.fabric.impl.attachment.AttachmentRegistryImpl;
import net.minecraft.class_10695;
import net.minecraft.class_10699;
import net.minecraft.class_10701;
import net.minecraft.class_10702;
import net.minecraft.class_2960;

public record MobVariant(
    Map<AttachmentType<?>, Object> components, class_10702 spawnConditions)
    implements class_10695<class_10701, class_10699> {

  private static final Codec<Map<AttachmentType<?>, Object>> COMPONENT_CODEC =
      Codec.dispatchedMap(
          class_2960.field_25139.comapFlatMap(MobVariant::getAttachment, AttachmentType::identifier),
          AttachmentType::persistenceCodec);

  public static final Codec<MobVariant> CODEC =
      RecordCodecBuilder.create(
          instance ->
              instance
                  .group(
                      COMPONENT_CODEC.fieldOf("components").forGetter(MobVariant::components),
                      class_10702.field_56286
                          .fieldOf("spawnConditions")
                          .forGetter(MobVariant::spawnConditions))
                  .apply(instance, MobVariant::new));

  private static DataResult<AttachmentType<?>> getAttachment(class_2960 id) {
    var attachmentType = AttachmentRegistryImpl.get(id);
    if (attachmentType == null) return DataResult.error(() -> "Attachment Type does not exist");
    if (attachmentType.persistenceCodec() == null)
      return DataResult.error(() -> "Attachment Type is not persistent", attachmentType);
    return DataResult.success(attachmentType);
  }

  @Override
  public List<class_10696<class_10701, class_10699>> method_67126() {
    return this.spawnConditions.comp_3583();
  }
}

==> mob_variants_api_zenxarch-0.0.2-sources.jar: org/codeberg/zenxarch/mob_variants_api/variant/MobVariant.java <==
package org.codeberg.zenxarch.mob_variants_api.variant;

import com.mojang.serialization.Codec;
import com.mojang.serialization.DataResult;
import com.mojang.serialization.codecs.RecordCodecBuilder;
import java.util.List;
import java.util.Map;
import net.fabricmc.fabric.api.attachment.v1.AttachmentType;
import net.fabricmc.fabric.impl.attachment.AttachmentRegistryImpl;
import net.minecraft.class_10695;
import net.minecraft.class_10699;
import net.minecraft.class_10701;
import net.minecraft.class_10702;
import net.minecraft.class_2960;
import net.minecraft.class_5699;
import net.minecraft.class_6880;
import net.minecraft.class_6899;
import org.codeberg.zenxarch.mob_variants_api.registry.MobRegistryKeys;

public record MobVariant(
    Map<AttachmentType<?>, Object> components,
    class_10702 spawnConditions,
    List<class_6880<MobVariant>> template)
    implements class_10695<class_10701, class_10699> {

  private static final Codec<Map<AttachmentType<?>, Object>> COMPONENT_CODEC =
      Codec.dispatchedMap(
          class_2960.field_25139.comapFlatMap(MobVariant::getAttachment, AttachmentType::identifier),
          AttachmentType::persistenceCodec);

  public static final Codec<class_6880<MobVariant>> ENTRY_CODEC =
      class_6899.method_40400(MobRegistryKeys.MOB_VARIANT);

  public static final Codec<MobVariant> CODEC =
      RecordCodecBuilder.create(
          instance ->
              instance
                  .group(
                      COMPONENT_CODEC.fieldOf("components").forGetter(MobVariant::components),
                      class_10702.field_56286
                          .fieldOf("spawnConditions")
                          .forGetter(MobVariant::spawnConditions),
                      class_5699.method_65313(ENTRY_CODEC)
                          .optionalFieldOf("template", List.of())
                          .forGetter(MobVariant::template))
                  .apply(instance, MobVariant::new));

  public MobVariant(
      Map<AttachmentType<?>, Object> components, class_10702 spawnConditions) {
    this(components, spawnConditions, List.of());
  }

  private static DataResult<AttachmentType<?>> getAttachment(class_2960 id) {
    var attachmentType = AttachmentRegistryImpl.get(id);
    if (attachmentType == null) return DataResult.error(() -> "Attachment Type does not exist");
    if (attachmentType.persistenceCodec() == null)
      return DataResult.error(() -> "Attachment Type is not persistent", attachmentType);
    return DataResult.success(attachmentType);
  }

  @Override
  public List<class_10696<class_10701, class_10699>> method_67126() {
    return this.spawnConditions.comp_3583();
  }
}
